##################################################################
##
## This file is for declaring the NSM pseudo-shell commands
## Please put them in block syntax like the following:
##
## command "<description>", "<command>", do |<argument_name>|
## <command execution>
## end
##
## The <description> is used when the 'help' command is called.
## <argument_name> is an array passed in by the commands args.
##
##################################################################

## Help command
command "Display commands and help", "help" do |args|
  puts " Available Commands:"
  puts "-" * 70
  CommandManager.get_commands.each { |k, val|
    print " #{val.exec}"
  }
  print "\n\n"
  puts " Command\t- Description"
  puts "-" * 70
  CommandManager.get_commands.each { |k, val|
    print " #{val.exec}"
    print " " * (15 - val.exec.length)
    print "- #{val.desc}\n"
  }
  print "\n"
  
end

## Quit/q command
command "Quit the program.", "quit" do |args|
  puts "Goodbye!"
  exit
end
command "Quit the program.", "q" do |args|
  puts "Goodbye!"
  exit
end
  
## List command
command "List all modules and their enabled or disabled states", "list" do |args|
  puts "\nCategories:"
  $categories.each { |cat|
    cat.dump
  }
  puts "--------------------------"
  puts "\nModules:"
  $modules.each { |mod|
    mod.dump
  }
  puts "--------------------------"
  puts "+ = Enabled\n- = Disabled"
end

## Toggle command
command "Toggle a module on or off\n\t\tUsage: toggle <module name>\n\t\tYou can also use \
'toggle all' and 'toggle none' to enable/disable all modules",
"toggle" do |name|
  name = name[0].to_s
  
  if name.length < 1
    puts "Need a module or category name"
    return
  end
  if name == "all"
    $modules.each { |mod| mod.set_enabled(true); }
    $categories.each { |cat| cat.set_enabled_mods(true); }
    puts "All modules and categories turned on."
    return
  elsif name == "none"
    $modules.each { |mod| mod.set_enabled(false); }
    $categories.each { |cat| cat.set_enabled_mods(false); }
    puts "All modules and categories turned off."
    return
  else
    $categories.each { |cat|
      if cat.get_name() == name
        if cat.enabled?
          cat.set_enabled_mods(false)
          puts "#{name} category turned off."
        else
          cat.set_enabled_mods(true)
          puts "#{name} category turned on."
        end
        return
      end
    }
    
    $modules.each { |mod|
      if mod.get_name() == name
        if mod.enabled?
          mod.set_enabled(false)
          puts "#{name} module turned off."
        else
          mod.set_enabled(true)
          puts "#{name} module turned on."
        end
        return
      end
    }
  end
  puts "Module '#{name}' not found."
  return
end

## Options command
command "Display global options. If <module> is given, display that module's \n\t\t\
options as well (ex: 'options hash'). 'options' will also display the commands \n\t\t\
that will be run (before substitution) for that module.",
"options" do |name|
  puts "\nGlobal options:"
  puts "-" * 35
  if File.directory?($datafile)
    puts "PCAP DIRECTORY: #{$datafile}"
    puts "Pcap basename will be determined individually"
  else
    puts "${PCAP_FILE}: #{$datafile}"
    puts "${PCAP_BASE}: #{$basefile}"
  end
  puts "${OUTPUT_DIR}: #{$outputdir}"
  puts "${MODULE_DIR}: #{$moduledir}"
  print "\n"
  
  name = name[0].to_s

  return if name.length < 1

  mod = get_mod_by_name(name)
  if mod.nil?
    puts "Unknown module #{name}"
    return
  end
  defs = mod.get_defaults
  puts "Options for module #{name}:"
  puts "-" * 35
  defs.each_pair { |d, val|
    puts "${#{d}} = #{val}"
  }
  cmds = mod.get_commands
  puts "\nCommand(s) to be executed for module #{name}:"
  puts "-" * 35
  cmds.each { |c| puts c }
  print "\n"
end

## File command
command "Set the ${PCAP_FILE} filename (the file to be analyzed).", "file" do |name|
  name = name[0].to_s

  if name.length < 1
    puts "Need a pcap filename"
    return
  end
  $datafile = name
  if File.directory?($datafile)
    puts "PCAP DIRECTORY: #{$datafile}"
    puts "Pcap basename will be determined by individual files."
  else
    puts "Setting ${PCAP_FILE} = #{$datafile}"
  end
  $basefile = `basename #{name}`.chomp
  puts "Setting ${PCAP_BASE} = #{$basefile}"
  print "\n"
end

## Output command
command "Set the ${OUTPUT_DIR} output directory.", "output" do |name|
  name = name[0].to_s
  
  if name.length < 1
    puts "Need a output directory name"
    return
  end
  puts "Setting ${OUTPUT_DIR} = #{name}"
  $outputdir = name
end

## Run command
command "Run enabled analysis files on the data (pcap) file.", "run" do |args|
      
  if $datafile == ""
    puts "Error: no pcap datafile has been specified"
    puts "specify one with 'file <file.pcap>'"
    return
  end
  
  if !File.exist?($datafile)
    puts "ERROR: #{$datafile} does not exist. Please set file using 'file <filename>'."
    return
  end
  
  fnames = []
  if File.directory?($datafile)
    puts "#{$datafile} is a directory, processing sub-files..."
    fnames = get_filelist($datafile,true)
    puts fnames
  else
    fnames << $datafile
  end
  
  puts "\nExecuting analysis...\n\n"
  
  
  $modules.each { |mod|
    if mod.enabled?
      puts "===> module #{mod.get_name} running..."
      fnames.each {
        |file|
        mod.run_commands(file)
      }
      puts "===> module #{mod.get_name} finished."
    else
      puts "===> module #{mod.get_name} skipped."
    end
    print "\n"
  }
end

## Info command
command "Show detailed module information.", "info" do |name|
  name = name[0].to_s
  
  mod = get_mod_by_name(name)
  if mod.nil?
    puts "Module not found."
    return
  end
  mod.print_info
  print "\n"
end

## Set command
command "Usage: set <mod> <opt> <val>\n\t\tSet the <mod> module's option \n\t\t\
'<opt>' to have the value <val>. (ex: 'set aimsnarf OUTPUT_FILE ${PCAP_BASE}.aim.txt').",
"set" do |args|
  args = args[0].to_s

  if args.length < 1
    puts "Need a module name."
    return
  end
  arglist = args.split(/ /, 3)
  mod = get_mod_by_name(arglist[0])
  if mod.nil?
    puts "Module #{arglist[0]} not found."
    return
  end
  if arglist[1].nil?
    puts "Need variable name (ex: OUTPUT_DIR)"
    return
  end
  if arglist[2].nil?
    puts "Need variable value (ex: file.pcap)"
    return
  end
  mod.update_option(arglist[1],arglist[2])
end

## Modload command
command "Load the modules from the given directory. Note that this replaces \n\t\t\
the currently loaded modules.",
"modload" do |dir|
  dir = dir[0].to_s
  
  if dir.length < 1
    puts "Please specify a module directory."
    return
  end
  
  if !File.directory?(dir)
    puts "#{dir} is not a directory."
    return
  end
  
  $moduledir = dir
  load_modules($moduledir)
  
  load_categories($moduledir)
  
end

## Exec command
command "Execute an external command", "exec" do |args|
  args = args[0].to_s
  
  if args.length < 1
    puts "Need a program to execute."
    return
  end
  
  cmd = args.gsub(/\$\{PCAP_FILE\}/i,$datafile)
  cmd.gsub!(/\$\{PCAP_BASE\}/i,$basefile)
  cmd.gsub!(/\$\{MODULE_DIR\}/i,$moduledir)
  cmd.gsub!(/\$\{OUTPUT_DIR\}/i,$outputdir)
  
  puts "Executing: #{cmd}"
  
  system(cmd)
  Logger.write("[exit: #{$?}] #{cmd.to_s}\n")
  puts "Exit status: #{$?}"
end
command "Execute an external command","e" do |args|
  CommandManager.execute("exec",args)
end

## Logfile command
command "Set the file to log commands to", "logfile" do |file|
  file = file[0].to_s
  
  if file.length < 1
    puts "Need a new logfile name."
    return
  end
  
  if File.directory?(file)
    puts "#{file} is a directory, can't log to that file."
    return
  end
  
  puts "New logfile is: #{file}"
  Logger.new(file)
  
  Logger.write("-" * 80)
  Logger.write("\n")
  Logger.write("Log for nsm-console begun at #{Time.now.year}-#{Time.now.month}-#{Time.now.day} #{Time.now.hour}:#{Time.now.min}:#{Time.now.sec}\n")
  Logger.write("-" * 80)
  Logger.write("\n")
  
end

## History command
command "Display command history", "history" do |args|
  History.print()
end

## Encode command
command "Encode value into different form\n\t\tRun without any options to see \
available formats",
"encode" do |args|
  args = args[0].to_s

  if args.length < 1
    puts "\nAvailable encoding:"
    puts "-" * 30
    Encoder.get_encode_list.each { |e| puts e; }
    return
  end
  arglist = args.split(/ /, 2)
  type = arglist[0].to_s
  str = arglist[1].to_s.chomp
  
  output = ""
  
  print "\n"
  puts "Encoding ascii --> #{type}...\n\n"
  puts "Output ([]'s added to show beginning and end):\n\n"
  
  begin
    cmd = "Encoder.encode_#{type}(\"#{str}\")"
    output = eval(cmd)
  rescue NoMethodError
    puts "Encoding not found."
  rescue
    puts "Error: #{$!}"
    return
  end
  
  print "["
  print output
  print "]\n\n"
  
end

## Decode command
command "Decode value into different ascii\n\t\tRun without any options to see \
available formats",
"decode" do |args|
    
  args = args[0].to_s

  if args.length < 1
    puts "\nAvailable decoding:"
    puts "-" * 30
    Encoder.get_decode_list.each { |d| puts d; }
    return
  end
  arglist = args.split(/ /, 2)
  type = arglist[0].to_s
  str = arglist[1].to_s.chomp

  output = ""
  
  print "\n"
  puts "Decoding #{type} --> ascii...\n\n"
  puts "Output ([]'s added to show beginning and end):\n\n"

  begin
    cmd = "Encoder.decode_#{type}(\"#{str}\")"
    output = eval(cmd)
  rescue NoMethodError
    puts "Decoding not found."
  rescue
    puts "Error: #{$!}"
    return
  end
  
  print "["
  print output
  print "]\n\n"
  
end

## Eval command
command "Evaluate a single line in Ruby","eval" do |args|
  args = args[0].to_s
  if args.length < 1
    puts "Need a line to eval."
    return
  end
  
  print "=> "
  begin
    ret = eval("#{args}")
  rescue
    puts "Caught error: #{$!}"
  end
  puts "#{ret}"
end

## Credits command :D
command "Display program credits","credits" do |args|
  print "\n"
  
  puts "Original design and development:"
  puts "-" * 40
  puts "Matthew Lee Hinman - <matthew [dot] hinman [at] gmail [dot] com"
  
  print "\n"
  puts "Ideas:"
  puts "-" * 40
  puts "Geek00l, Enhanced, Scholar"

  print "\n"
end

